<?php

namespace App\Service;

use App\Entity\Note;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class AnkiService
{
    function __construct(
        private HttpClientInterface $client,
    ) {}

    private function request(string $action, array $params): mixed
    {
        ['error' => $error, 'result' => $result] = $this->client->request(
            'POST',
            'http://localhost:8765',
            ['json' => [
                'action' => $action,
                'version' =>  6,
                'params' => $params,
            ]]
        )->toArray();

        if ($error != null) {
            throw new \Exception('AnkiConnect returned error: ' . $error);
        }

        return $result;
    }

    public function getAllNoteIds(): array
    {
        return $this->request(
            'findNotes',
            ['query' => '"note:Japanese sentences" -is:suspended'],
        );
    }

    /** Give an array of IDs, the note Infos are returned. if info for a given
     *  doesn't exist, it is assigned to null instead of the default [].
     */
    private function getNotesInfo(array $noteIds): array
    {
        $noteInfos = $this->request('notesInfo', ['notes' => $noteIds]);

        foreach ($noteInfos as &$noteInfo) {
            if ([] === $noteInfo) $noteInfo = null;
        }

        return $noteInfos;
    }

    /** Returns info form note given an ID, returns null if it doesn't exist */
    private function getNoteInfo(int $noteId): ?array
    {
        return $this->getNotesInfo([$noteId])[0];
    }

    public function getNote(int $nid): ?Note
    {
        return Note::fromAnki($this->getNoteInfo($nid));
    }

    public function getLatestNote(): ?Note
    {
        // NoteIDs are just timestamps in milliseconds, so the latest is just
        // the biggest numerically
        $latestId = max($this->getAllNoteIds());
        return $this->getNote($latestId);
    }

    public function updateNote(Note $note)
    {
        $this->request('guiBrowse', ['query' => 'nid:1']);
        $this->request('updateNoteFields', ['note' => $note->toAnki()]);
        $this->request('guiBrowse', ['query' => 'nid:' . $note->getId()]);
    }
}
